% P52SNRpileup5.m Constant Covariance Approximation
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2015. All rights reserved.
 
%% init vars
post_title = 'P52SNRpileup5';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
% MakeBlogPostFileLists(post_title);

%% compare CRLB at 0 pileup with usual formula
    % Create the input spectrum and attenuaton coefficients
tube_voltage = 80; % keV  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
materials = {'soft_tissue','bone_cortical'}; 
specdat.mus = MusFromMaterials(materials,specdat.egys);
     % optimal PHA thresholds 
nbins = 3;     
specdat.idx_threshold = OptimNbinsThresholds(specdat,nbins);
    % A-vectors of points on line through A-space--use complex notation
npoints = 10;
zAs = linspace(0,complex(20,1),npoints).'; %  evenly spaced from (soft tissue,bone)= (0,0) to (20,1) 
    % Mmatrices and nphotons/bin (lambdas) for CRLB no pileup
[Ms,~] =  mubarNbinNdim(specdat,zcomp(zAs)); % Ms has M for each point on line
lambdas = PoissonLambda(zAs,specdat);
CRLB_no_pileup = zeros(2,2,npoints); % pre-allocate for full CRLB incl change in covariance
CRLB_with_pileup =   zeros(2,2,npoints);
CRLBCC_no_pileup = zeros(2,2,npoints); % pre-allocate for constant covariance approximation
CRLBCC_with_pileup =   zeros(2,2,npoints);
spectran = specdat; % transmitted spectrum for this point on line
for kpnt = 1:npoints
    RL = diag(1./lambdas(kpnt,:)); % covariance of count data
       % no pileup CRLB
    [CRLB_no_pileup(:,:,kpnt),mdat] = CRboundNoLinearize(RL,Ms(:,:,kpnt));
    CRLBCC_no_pileup(:,:,kpnt) = mdat.CR_const_cov;
       % with pileup CRLB--uses transmitted spectrum for each point in A space
    spectran.specnum = specdat.specnum.*exp(-specdat.mus*zcomp(zAs(kpnt))');
    [CRLB_with_pileup(:,:,kpnt),stats] = CRboundNoLinearizeWithPileup(spectran,1,0); % zero deadtime
    CRLBCC_with_pileup(:,:,kpnt) = stats.CR_const_cov;
end

%% plot the variances vs. dist from origin in A-space
fh = figure;
subplot(121)
hp1 = plot(abs(zAs),squeeze([CRLB_with_pileup(1,1,:) CRLB_no_pileup(1,1,:)]),'linewidth',2);
v = axis;
axis([0 20 v(3:4)])
set(hp1(1),'color','r')
set(hp1(2),'color','k')
xlabel('thickness (gm/cm^2)','fontsize',14);
ylabel('\sigma_{A_1}^2','fontsize',14);
set(gca,'fontsize',14,'linewidth',2,'yscale','log');
hl = legend('pileup formula','no pileup formula','location','southeast');
title('full CRLB','fontsize',14)

subplot(122)
hp2 = plot(abs(zAs),squeeze([CRLBCC_with_pileup(1,1,:) CRLBCC_no_pileup(1,1,:)]),'linewidth',2);
axis([0 20 v(3:4)])
xlabel('thickness (gm/cm^2)','fontsize',14);
set(gca,'fontsize',14,'linewidth',2,'yscale','log');
hl = legend('pileup formula','no pileup formula','location','southeast');
title('constant covariance CRLB','fontsize',14)

set(fh,'color','w')

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'CompareVarFormulas';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% const cov CRLB error with pileup
npoints_on_line=  75;% number of points along line
Nphotons = linspace(10,1000,npoints_on_line);
n_etas = 4;
etas = linspace(0,0.5,n_etas);
rate = 2e7;
CR_error_frac_pileup = zeros(npoints_on_line,n_etas,2); % matrix to hold results : 3d dim==PHA, NQ
var_CR_with_pileup = zeros(npoints_on_line,n_etas,2); % variance for each component of A-vector
for keta = 1:n_etas
    eta = etas(keta);
    deadtime = eta/rate;
    for kt = 1:npoints_on_line
        sp = specdat;
        sp.specnum = Nphotons(kt)*sp.specnum_norm;
        Tintegrate = Nphotons(kt)/rate;
        [RA_CR,pha_stats] = CRboundNoLinearizeWithPileup(sp,Tintegrate,deadtime);
        CR_error_frac_pileup(kt,keta,1) = norm(RA_CR - pha_stats.CR_const_cov)/norm(RA_CR); 
        var_CR_with_pileup(kt,keta,:) = diag(RA_CR);
        [RA_NQ,nqstats] = CRboundNQNoLinearizeWithPileup(sp,Tintegrate,deadtime);
        CR_error_frac_pileup(kt,keta,2) = norm(RA_NQ - nqstats.CR_const_cov)/norm(RA_NQ); 
    end
end
fprintf('finished\n');

%% plot frac error vs number of photons and eta--with pileup
fh = figure;
linewid = 1.5;
colors = 'krbgm';
plot_symbols = {'-o','-x','-*','-d','-+'};
plot_order = n_etas:-1:1; % 1:n_etas; %use this so legends in same order as plots
subplot(211)
hold on
for keta = plot_order
    semilogx(Nphotons,CR_error_frac_pileup(:,keta,1),[plot_symbols{keta} colors(keta)],'linewidth',linewid);
end
hold off
axis([10 1000 0 1])
set(gca,'xscale','log','fontsize',14,'linewidth',1.5)
ht1 = text(100,0.7,'incident counts \newlineper deadtime','fontsize',12);
ht2 = text(20,0.8,'3-bins PHA','fontsize',14);
eta_strings = strtrim(cellstr(num2str(etas(plot_order(:))','%4.2f')));
hl = legend(eta_strings,'location','east');
hy = ylabel('norm( CRLB-CRLB_{const. cov} )/norm(CRLB)','fontsize',13);
   % tweak positions of text and legend
set(hl,'position',[0.68 0.61 0.2153 0.1234])
set(ht1,'position',[200 0.6 0])

subplot(212)
hold on
for keta = plot_order
    semilogx(Nphotons,CR_error_frac_pileup(:,keta,2),[plot_symbols{keta} colors(keta)],'linewidth',linewid);
end
axis([10 1000 0 1])
set(gca,'xscale','log','fontsize',14,'linewidth',1.5)
% ht = text(125,0.8,'\eta = rate*T_{deadtime}','fontsize',12);
ht = text(20,0.8,'NQ','fontsize',14);
xlabel('number of photons','fontsize',14)
   % position ylabel so it can be used for both coord axes
set(hy,'position',[7    0    1.1])
set(fh,'color','w','position',[250 100 500 850])

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'CRLBvsConstCovWithPileup';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',5,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

