function spout = SpectrumWithPileup(sp,eta,varargin)
% function spout = SpectrumWithPileup2(sp,eta,varargin)
% This version uses an alternate approach. Has a fixed order 
% and then combines orders using weights computed from eta
% previous
%    create spectrum probability distribution function (PDF) after pileup by convolution
%  inputs:
%      sp: the input spectrum: a spectrum struct with fields
%          egys: the photon energies (keV)--assumed equally spaced
%          specnum: the number spectrum
%      eta: mean_rate*deadtime
%      maxorder: (optional: 10) max pileup order in calculations
%   outputs:
%      spout = convolved spectrum struct with same fields as above but added fields for debugging
% The output has same number of counts as input
% The energy range will be increased but same energy spacing
%  REA 2011-Jul-23 11:22 - 2012-May-2 16:26
%  COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

nreqargs = 2;
assert(nargin>=nreqargs);
maxorder = 10;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'maxorder';         maxorder=varargin{i+1};   assert(isscalar(maxorder));  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if abs(eta)<5*eps
    spout = sp;
    spout.eta = eta;
    spout.spec_in = sp;
    spout.maxorder = 0;
    return
end
   % compute spectra for all orders
       % normalize input spectrum to make pdf
lambda = trapz(sp.egys,sp.specnum); 
spnorm = sp.specnum/lambda;

    % min and max energy for convolved spectra
minE = min(sp.egys);
maxE = max(sp.egys);

    % energy range for convolved spectra
deltaE = (sp.egys(end) - sp.egys(1))/(numel(sp.egys) - 1);
egys = (minE:deltaE:(maxorder + 1)*maxE)'; % transpose for column vector
negys = numel(egys);
    % put input spectrum into 1st row of spectra
spectra = zeros(maxorder+1,numel(egys));
spectra(1,1:numel(spnorm)) = spnorm(:)';

    % now the convolved spectra
    % initialize convolved spectrum to normalized input spectrum
spconv = spnorm; % holds repeated convolutions of spnorm
norms = zeros(maxorder+1,1);
dE = egys(2) - egys(1);
if abs(dE -1) > 10*eps
    warning('SpectrumWithPileup dE ~= 1');
end
norms(1) =  sum(spnorm)*dE;
for k = 1:maxorder
   fc = conv(spnorm,spconv);
       % put convolved data into correct location in output array
%    idxsdebug = round(interp1(egys,1:negys, linspace((k+1)*minE, (k+1)*maxE, numel(fc)))  );
   idx1 = find(egys >= (k+1)*minE,1,'first');
   idxs = idx1:(idx1+numel(fc)-1);
   spectra(k+1,idxs) = fc;
   norms(k+1) = sum(fc)*dE;
   spconv = fc/norms(k+1); % has unit norm
end

       % create output spectrum
spout.egys = egys(:);
weights = poisspdf(0:maxorder, eta);
spout.specnum = weights*spectra;
   % set expected output counts per non-paralyzable formula
spout.specnum = (lambda/(1 + eta))*spout.specnum/trapz(egys,spout.specnum); 
   % additional information
spout.egys = spout.egys(:);
spout.specnum = spout.specnum(:);
spout.spec_in = sp;
spout.spectra = spectra;
spout.norms = norms;
spout.maxorder = maxorder;
spout.weights = weights;
spout.eta = eta;
