function r = zcomp(zs)
% function r = zcomp(zs)
%  returns a 2 column matrix of the real and imaginary components of a complex vector
% else returns a complex vector if input  is a 2 column matrix
% in other words, this function works both ways, either extracts real/imag
% components or converts a 2 col matrix to complex vector
%  inputs:
%    zs: the vector or 2 columnmatrix
%  outputs
%    r: [real(zs(:)) imag(zs(:))]
%  REA 4/26/10 - 2011-Aug-10
%  COPYRIGHT:   Aprend Technology, 2011. All rights reserved.
% see accompanying license.txt

if isempty(zs)
   r = [];
   return
end

switch numel(zs)
    case 1; r = [real(zs(:)) imag(zs(:))];
    case 2; 
        if isreal(zs) 
           r = complex(zs(1),zs(2));
        else
           r = [real(zs(:)) imag(zs(:))];
        end
    otherwise
        if isvector(zs)
           r = [real(zs(:)) imag(zs(:))];
        elseif size(zs,2) == 2
            r = complex(zs(:,1),zs(:,2));
        else
            error('input must be either a vector of complex or a 2 column matrix');
        end
end